class Thing	
	attr_accessor( :name, :description )
	 
	def initialize( aName, aDescription )
		@name = aName 
		@description = aDescription
	end
end

class Treasure < Thing	
	attr_accessor( :value )
	
	def initialize( aName, aDescription, aValue )
		super( aName, aDescription )
		@value = aValue
	end
end

class Room < Thing	
	attr_accessor( :n, :s, :w, :e )
	
	def initialize( aName, aDescription, dirN, dirS, dirW, dirE )
		super( aName, aDescription )		
		@n = dirN
		@s = dirS
		@w = dirW
		@e = dirE
	end
end

class Actor < Thing
	attr_accessor( :position )
	
	def initialize( aName, aDescription, aPosition )
		super( aName, aDescription )
		@position = aPosition
	end
end

class Map	
	attr_reader( :rooms )
	
	def initialize( someRooms )
		@rooms = someRooms
	end				
end

class Game
	attr_accessor( :map )
	
	def initialize( someRooms, player )
		@map = Map.new( someRooms )
	end		
end

# === The Map ===
# @r0 -- @r1
#   |
# @r2 -- @r3

class Implementer
	attr_accessor( :game )
	
	def initialize( aPlayer )
		# if you want to make this 'game system' more generic, you could load the data 
		# from a file instead of 'hard coding' it into the program
																	 # N   S   W   E
		@r0 = Room.new("Treasure Room", "a fabulous golden chamber",  -1,  2, -1,  1)
		@r1 = Room.new("Dragon's Lair", "a huge and glittering lair", -1, -1,  0, -1)
		@r2 = Room.new("Troll Cave", "a dank and gloomy cave",         0, -1, -1,  3)
		@r3 = Room.new("Crystal Dome", "a vast dome of glass",        -1, -1,  2, -1)	
		@player = aPlayer
		@game = Game.new( [@r0,@r1,@r2,@r3], @player  )	
	end
	
	def moveActorTo( anActor, aDirection )
		reply = "No Exit!"
		exit = @game.map.rooms[anActor.position].method(aDirection).call
		if (exit > -1) then
			anActor.position = exit
			reply = "You have entered the #{@game.map.rooms[exit].name} which is #{@game.map.rooms[exit].description}."
		end
		return reply
	end
end

# Test it out. Create the player and the game
# Then simulate some moves around the map
thePlayer = Actor.new("The Player", "You", 0 )
imp = Implementer.new( thePlayer )
puts( imp.moveActorTo( thePlayer, :e ) )
puts( imp.moveActorTo( thePlayer, :w ) )
puts( imp.moveActorTo( thePlayer, :n ) )
puts( imp.moveActorTo( thePlayer, :s ) )
puts( imp.moveActorTo( thePlayer, :e ) )
puts( imp.moveActorTo( thePlayer, :s ) )
